<?php

/*
Plugin Name: Wordpress Forms
Plugin URI: http://wordpress.org/extend/plugins/wordpress-forms
Version: 0.2.4
Author: Vitor Carvalho
Author URI: http://lightningspirit.net
Description: Create your Web forms quickly and intuitively, and use them within your Posts, Pages or even published in a single location.
Tags: plugin, forms, ui, gui, form builder, custom forms, easy forms, drag and drop, form creator, form manager, jquery form, wordpress cms, custom post types, form in, posts, pages
License: GPL2

*/




/*
 *      Wordpress Forms
 *      
 *      Copyright 2010 Vitor Carvalho <lightningspirit@gmail.com>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */
 



/* Release History :
 * 0.2.4:	Bug Fix: Hiding print_r in editor-plugin.php
 * 0.2.3:	Adding tinyMCE button functionality
 * 0.2.2:	Adding tinyMCE button functionality
 * 0.2.1:	Version Control Fix
 * 0.2:	Bug Fixes: dynamic folders
 * 0.1:	Initial Release
 * 
 */








// Wordpress version control. No compatibility with older versions. ( wp_die )
if ( version_compare( get_bloginfo( 'version' ), '3.0.0', '<' ) ) {
	wp_die( 'Wordpress Forms can be used only with Wordpress 3.0 version' );

}


// PHP version control. No compatibility with older versions. ( wp_die )
if ( version_compare(PHP_VERSION, '5.0.0', '<') ) {
	wp_die( 'Wordpress Forms can be used only with PHP 5.+ version' );

}





/* Define Constants to be used within plugin */

// Path to the plugin directories
define( 'WP_FORMS_PATH', dirname( __FILE__ ) . '/' );
define( 'WP_FORMS_LIBS', WP_FORMS_PATH . 'libraries/' );
define( 'WP_FORMS_LANGS', WP_FORMS_PATH . 'langs/' );


// URL to some needed directories
define( 'WP_FORMS_URL', plugins_url( basename( dirname( __FILE__ ) ) ) );
define( 'WP_FORMS_JS', WP_FORMS_URL . '/js' );
define( 'WP_FORMS_CSS', WP_FORMS_URL . '/css' );
define( 'WP_FORMS_LOC', WP_FORMS_URL . '/langs' );
define( 'WP_FORMS_IMG', WP_FORMS_URL . '/images' );
define( 'WP_FORMS_ICONS', WP_FORMS_URL . '/images/icons' );
define( 'WP_FORMS_IMGLANG', WP_FORMS_URL . '/images/translations' );



// Max upload file size from php.ini - value in MB
if ( ! defined( 'MAX_UPLOAD_SIZE' ) ) {
	define('MAX_UPLOAD_SIZE', min( (int) ( ini_get( 'upload_max_filesize' ) ), (int) ( ini_get( 'post_max_size' ) ) ) );

}



// Include form button for WYSIWYG editor
require_once( WP_FORMS_PATH . 'editor-plugin.php' );










/*
 * 
 * name: (class) WP_Form - Form builder and manage "Custom Post Type" powered.
 * @param none
 * @return none
 */
class WP_Form
{

	
	// Init the Register Post Type and Calls other functions
	function WP_Form() { 
		$labels = array(
			'name'			=> __( 'Forms', "wp_forms" ), // Tip: _x('') is used for localization
			'singular_name' 	=> __( 'Form', "wp_forms" ),
			'add_new' 		=> __( 'Create New', "wp_forms" ),
			'add_new_item' 	=> __( 'Create New Form', "wp_forms" ),
			'edit_item' 		=> __( 'Edit Form', "wp_forms" ),
			'new_item' 		=> __( 'New Form', "wp_forms" ),
			'view_item' 		=> __( 'View Form', "wp_forms" ),
			'search_items' 	=> __( 'Search Form', "wp_forms" ),
			'not_found' 		=> __( 'No Forms found', "wp_forms" ),
			'not_found_in_trash'=> __( 'No Forms found in Trash', "wp_forms" ),
			'parent_item_colon' => ''
		);

		// Create an array for the $args
		$args = array(
			'labels' 			=> $labels,
			'description' 		=> __( 'Create your Web forms quickly and intuitively, and use them within your Posts, Pages or even published in a single location.' ),
			'public' 			=> true,
			'publicly_queryable' => true,
			'exclude_from_search' => true,
			'show_ui' 			=> true,
			'query_var' 		=> true,
			'rewrite' 			=> array( 'slug' => 'form', 'with_front' => true ),
			'capability_type' 	=> 'page',
			'hierarchical' 		=> false,
			'menu_position' 	=> null,
			'supports' 			=> array( 'title', 'author', 'revisions' ),
			'taxonomies' 		=> array(),
			'menu_icon' 		=> WP_FORMS_ICONS . '/application_form_add.png',
			
		); 
		
		// Regist Our Form type
		register_post_type( 'form', $args );
		
		// Load plugin i18n
		load_plugin_textdomain( 'wp_forms', null, WP_FORMS_PATH . 'lang' );
		
		// Refresh Permalinks
		flush_rewrite_rules();
		
		
			
		// Admin interface init - Loads everything required for the UI to work
		add_action( "admin_init", array( &$this, "admin_init" ) );
		
		
			
		// Insert / Update post hook
		add_action( "wp_insert_post", array( &$this, "wp_insert_post" ), 10,2 );
		add_filter( "wp_insert_post_data" , array( &$this, "post_data_filter" ), 10,2 );
		
		
		
		/* AJAX Calls from our plugin */
		
		// User call for new element inside admin area
		add_action( "wp_ajax_create_form_element", array( &$this, "create_element" ) );
		
		// Outputs shortcode [form id=...] inside WYSIWYG editor
		add_action( "wp_ajax_select_form_for_editor", array( &$this, "select_form_for_editor" ) );
		
		// Form frontend submition is always AJAX
		add_action( "wp_ajax_wp_forms_submit_form", array( &$this, "process_submition" ) );
		
		
		
		
		// Print Frontend Styles and Scripts required
		add_action( "wp_print_scripts", array( &$this, "frontend_head" ) );
		add_action( "wp_print_styles", array( &$this, "frontend_head" ) );
	
	
	}
	
	
	
	
	
	/*
	 * 
	 * name: process_submition() - Processes frontend form submitions through AJAX call.
	 * @param none
	 * @return none
	 */
	function process_submition() { 
		
		// Lets make validations
		if ( empty( $_POST ) or ! wp_verify_nonce( $_POST['_wpnonce'], 'wp-form-nonce' ) ) {
			wp_die( 'Validation not passed!' );
			
		}
	
		// Retrieve form information with form ID that will be sent to the email
		$form = new WP_Query( array( 'p' => $_POST['form_id'], 'post_type' => 'form' ) );
		
		
		// If any form, go on!
		if ( $form->have_posts() ) {
			
			$form->the_post();
			
			// Retrieve form metadata
			$meta = get_post_custom(get_the_ID());
			$elements = unserialize($meta['_elements'][0]);
			
			// Sets email To: parameter
			$email_to = empty( $meta['_form_email'][0] ) ? get_option( 'admin_email' ) : $meta['_form_email'][0];
			
			// Email content
			$message = "";
			$attachments = array();
			
			// Retrieve our elements IDs
			$field_keys = unserialize( str_replace('\\', '', html_entity_decode($_REQUEST['field_keys'])) );
			
			
			
			// Process fileuploads
			while( list( $key, $field ) = @each( $_FILES ) )
			{
							
				if ( $_FILES[$key]['size'] > MAX_UPLOAD_SIZE * 1024 * 1024 )
					wp_die( "Error: Your file can't exceed " . MAX_UPLOAD_SIZE . " MB." );
				
				if ( $_FILES[$key]['error'] )
					wp_die( "Error: " . $_FILES['file']['error'] );
					
				if ( $_FILES[$key]['type'] == 'application/octet-stream' or $_FILES[$key]['type'] == 'application/x-httpd-php' )
					wp_die( "Error: For security reasons you can't upload application files!" );
				
				
				$upload_dir = wp_upload_dir();
				move_uploaded_file( $_FILES[$key]['tmp_name'], $upload_dir['path'] . '/' . $_FILES[$key]['name'] );
				$attachments[] = $upload_dir['path'] . '/' . $_FILES[$key]['name'];			
			}
			
			
			// Process POST requests
			while( list( $key, $field ) = @each( $_POST ) )
			{
				// If current is an hidden field move on
				if ( $key == 'action' or $key == 'form_id' or $key == 'field_keys' or $key == 'MAX_FILE_SIZE' or $key == '_wpnonce' or $key == '_wp_http_referer' ) continue;
				
				// Gets the real element ID
				$key = $field_keys[$key];
				
				// Checks if is any email in the form, that email will be used as the sender of the message
				if ( $elements[$key]['type'] == 'email' )
					$email_from = strip_tags( $field );
						
				// Build body
				$message .= "<b>{$elements[$key]['label']}</b> ";
					
				// If the field has multiple values	- array, like Name, Checkboxes, etc...
				if (is_array( $field )) {
					
					// Put it accordingly - default: separated with commas
					
					if ( $elements[$key]['type'] == 'name' ) {
						$name_from = strip_tags( $field[0] ) . " " . strip_tags( $field[1] );
						$message .= $name_from;
						
					}
					
					else {
						
						foreach( $field as $f ) {
							$message .= strip_tags( $f ) . ", ";
						}
						
						$message = substr($message,0,-2);
						
					}
				}
				
				// If has just one value - nl2br for textareas
				else {
					$message .= nl2br(strip_tags( $field ));
					
				}
				
				// New line					
				$message .= "<br/>";
					
			
			}
			
			
			// If we've got a Name and Email input field... lets "headers" that information
			if ( isset( $email_from ) AND isset( $name_from ) ) {
				$headers = "From: $name_from <$email_from>\r\n";
				$headers .= "Reply-To: $name_from <$email_from>\r\n";
				
			}
			
			// or use our Wordpress defaults
			else {
				$headers = "From: " . get_option( 'blogname' ) . " <wordpress@" . $_SERVER['HTTP_HOST'] . ">\r\n";
				
			}		
			
			$headers .= "MIME-Version: 1.0\r\n";
			$headers .= "Content-Type: text/html; charset=" . get_option( 'blog_charset' ) . "\r\n";
			
			// Send Email
			wp_mail( "Administrator <$email_to>", "[Form Submition] " . get_the_title(), $message, $headers, $attachments );
			
			die( __("<span class='sucess'>Your form was sent!</span>") );
		
		}
		else {
			wp_die( __("There was an error submiting your form!") );
			
		}
 
	}
	
	
	
	
	
	
	
	/*
	 * 
	 * name: select_form_for_editor() - AJAX return form to the WYSIWYG editor
	 * @param none
	 * @return none
	 */
	function select_form_for_editor() { 
	
		// Query for published forms
		$form = new WP_Query( array( 'post_type' => 'form' ) );

		$output = "";
		
		if($form->have_posts())
		{
			$output .= "<select name='select_form' id='select_form' style='width:100%'>";
		
			while($form->have_posts()){
			
				$form->the_post();
				
				$output .= "<option value='".get_the_ID()."'>".get_the_title()."</option>";
				
			}

			$output .= "</select>";
		
		}
		else
		{
			$output .= "<p>No Forms Found</p>";
		}
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en-US">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
	<script type="text/javascript" src="<?php bloginfo('url'); ?>/wp-includes/js/tinymce/tiny_mce_popup.js?ver=0.1"></script>
	<script type="text/javascript" src="<?php echo WP_FORMS_JS; ?>/wpformsdialog.js?ver=0.1"></script>
	<title>Insert New Form</title>
	<link rel="stylesheet" href="<?php bloginfo('url'); ?>/wp-admin/css/global.css" type="text/css" media="all" />
	<link rel="stylesheet" href="<?php bloginfo('url'); ?>/wp-admin/css/colors-classic.css" type="text/css" media="all" />
</head>

<body style="display:none; overflow:hidden;">

<div id="img-edit">
<form onsubmit="return WPFormsDialog.insert();" action=#">

	<div style="margin:15px;padding:15px;border:1px solid #ccc;">
	
		<img src="<?php echo WP_FORMS_IMG; ?>/form.png" alt="" style="float:right;" />
		<p>Select a Form to insert:</p>
		
		<p id="sel_form"><?php echo $output; ?></p>
					
	</div>


	<div id="saveeditimg" style="margin:15px;">
		<input type="submit" id="saveimg" class="button" value="Insert" onClick="submit();" />
		<input type="button" class="button" id="cancelimg" name="cancelimg" value="Cancel" onclick="tinyMCEPopup.close();" />
	</div>
</form>
</div>

</body>
</html>
<?php
		
		die();
	}
	

	
	
	




	/*
	 * 
	 * name: frontend_head() - Prints Scripts and Styles for the Form
	 * @param none
	 * @return none
	 */
	function frontend_head() { 
	
		// Limit it to the most non-obstrutive possible
		if( !is_page() AND !is_single() AND !is_front_page() ) return false;
		
		// Get Locale
		global $locale;
		
		
		
		// Include jQuery
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'jquery-form' );
		
		
		// Include Validation
		wp_enqueue_script( 'jquery-validate', WP_FORMS_JS . '/jquery.validate.min.js', array( 'jquery' ), '1.5.5' );
		wp_enqueue_script( 'jquery-validate-messages', WP_FORMS_JS . "/localization/{$locale}.js", array( 'jquery-validate' ), '1.5.5' );
		wp_enqueue_script( 'jquery-metadata', WP_FORMS_JS . '/jquery.metadata.min.js', array( 'jquery' ), '1.5.5' );
		
		
		// Include Form
		wp_enqueue_script( 'wp-form', WP_FORMS_JS . '/frontend.form.js', array( 'jquery-form', 'jquery-validate' ), '0.1.0' );
		wp_localize_script( 'wp-form', 'wp_form_strings', array(
			'url' => get_bloginfo( 'url' ),
			'img' => WP_FORMS_IMG,
			'icons' => WP_FORMS_ICONS,
		));
		
		
		
		// Include some Style
		wp_enqueue_style( 'wp-form', WP_FORMS_CSS . '/frontend.form.css', "", '0.1.0', 'screen' );


		/* Theme HACKS */
		
		// Submit Exceptions for default (and official) Themes
		switch ( get_current_theme() ) {
			
			case 'Twenty Ten' :
				wp_enqueue_style( 'wp-form-tth', WP_FORMS_CSS . '/twentyten-hack.css', "", '0.1.0', 'screen' );
				break;
				
			default : break;
			
		}
		
	}
	
	






	/*
	 * 
	 * name: admin_init() - Compose the Admin page for Forms
	 * @param none
	 * @return none
	 */
	function admin_init() { 
	
		// Limit the printing of scripts and styles
		if( ! isset( $_GET['post'] ) AND ( isset( $_GET['post_type'] ) AND $_GET['post_type'] != 'form' ) ) 
			return false;
		
		
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'jquery-ui-sortable' );
		wp_enqueue_script( 'wp-form-admin', WP_FORMS_JS . '/admin.form.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-dialog', 'jquery-ui-sortable' ), '0.1.0' );
		wp_localize_script( 'wp-form-admin', 'wp_form_strings', array(
			'url' => get_bloginfo( 'url' ),
			'img' => WP_FORMS_IMG,
			'icons' => WP_FORMS_ICONS,
		));

		wp_enqueue_style( 'jquery-ui', WP_FORMS_CSS . '/smoothness/jquery-ui-1.8.5.custom.css', "", '0.1.0', 'all' );
		wp_enqueue_style( 'wp-form-admin', WP_FORMS_CSS . '/admin.form.css', "", '0.1.0', 'all' );
		
		
		
		// Custom meta boxes for the edit Form screen
		add_meta_box( 'forms-main-box', __('Edit Form', "wpforms" ), array( &$this, "main_box" ), "form", "normal", "high");
		add_meta_box( 'forms-options-box', __('Form Options', "wpforms" ), array( &$this, "options_box" ), "form", "normal", "high");
		add_meta_box( 'forms-side-elements', __('Form Elements', "wpforms" ), array( &$this, "side_elements" ), "form", "side", "low");

	}








	/*
	 * 
	 * name: wp_insert_post() - When a form is inserted or updated
	 * @param $post_id
	 * @param $post
	 * @return none
	 */
	function wp_insert_post( $post_id, $post = null ) { 
	

		if ( $post->post_type == 'form' ) {
			
			$action = ( isset( $_POST['_form_action'] ) ) ? $_POST['_form_action'] : false;
			$method = ( isset( $_POST['_form_method'] ) ) ? $_POST['_form_method'] : false;
			$email 	= ( isset( $_POST['_form_email'] ) ) ? $_POST['_form_email'] : false;
			$elements = ( isset( $_POST['elements'] ) ) ? $_POST['elements'] : false;
			$order 	= ( isset( $_POST['elements_order'] ) ) ? $_POST['elements_order'] : false;
			
			// Sanitizes and updates meta-values
			$action = filter_var( strip_tags( $action ), FILTER_SANITIZE_URL );
			$method = filter_var( strip_tags( $method ), FILTER_SANITIZE_STRING );
			$email 	= filter_var( strip_tags( $email ), FILTER_SANITIZE_EMAIL );
			
			update_post_meta( $post_id, '_form_action', $action);
			update_post_meta( $post_id, '_form_method', $method);
			update_post_meta( $post_id, '_form_email', $email);
			
			update_post_meta( $post_id, '_elements', $elements);
			update_post_meta( $post_id, '_elements_order', $order);

		}

	}
	
	
	
	
	
	
	
	
	
	/*
	 * 
	 * name: post_data_filter() - When a form is inserted or updated - Stores HTML form
	 * @param $data
	 * @param $postarr
	 * @return $data
	 */
	function post_data_filter($data, $postarr=null) {
			
		global $post;
		
		if ( ! is_object( $post ) or $post->post_type != 'form' )
			return $data;


		//Cannot do this because it is updated after content: $custom = get_post_custom($post->ID); 
		$action 	= isset( $_POST['_form_action'] ) ? ( ! empty($_POST['_form_action'] ) ? $_POST['_form_action'] : admin_url( 'admin-ajax.php' ) ) : false; //$custom['_form_action'][0];
		$method 	= isset( $_POST['_form_method'] ) ? $_POST['_form_method'] : false; //$custom['_form_method'][0];
		$email 		= isset( $_POST['_form_email'] ) ? ( empty( $_POST['_form_email'] ) ? false : $_POST['_form_email'] ) : false;
		$elements 	= isset( $_POST['elements'] ) ? $_POST['elements'] : false; //unserialize($custom['_elements'][0]);

		ob_start();
		
?>
<form id="form-<?php echo $post->ID; ?>" name="form-<?php echo $post->ID; ?>" class="wp_form" enctype="multipart/form-data" action="<?php echo $action; ?>" method="<?php echo $method; ?>">
	<fieldset>
		<ul class="wp_form">
			<?php echo $this->build( $elements ); ?>
		</ul>
	</fieldset>
	<input type="hidden" name="action" value="wp_forms_submit_form" />
	<input type="hidden" name="form_id" value="<?php echo $post->ID; ?>" />
	<?php wp_nonce_field('wp-form-nonce'); ?>
</form>
<?php			

		$content = ob_get_contents();

		$data['post_content'] = $content;
		$data['post_content_filtered'] = $content;
		$data['to_ping'] = "";
		
		return $data;
		
	}
	
	






	/*
	 * 
	 * name: build() - When a form is inserted or updated - Builds HTML form
	 * @param $elements
	 * @return $html
	 */	
	function build( $elements ) { 
		
		// Require build helper library
		require_once("libraries/build.helper.php");
		
		$html = "";
		
		
		if ( ! is_array( $elements ) )
			return false;
			
			
		
		$field_keys = array();
		
		
		foreach( $elements as $key => $element )
		{
			
			if ( $element['type'] == 'paragraph' ) { 
				$html .= form_paragraph( $element['value'] );
			}
			
			else {
			
				$html .= "<li>";
							
				switch( $element['type'] )
				{
			
					case 'text': $html .= form_text( $element['label'], $element['value'], $element['size'], $element['required'], $element['name'] ); break;
					case 'email': $html .= form_email( $element['label'], $element['value'], $element['size'], $element['required'], $element['name'] ); break;
					case 'number': $html .= form_number( $element['label'], $element['value'], $element['size'], $element['required'], $element['name'] ); break;
					case 'name': $html .= form_name( $element['label'], $element['values'], $element['size'], $element['required'], $element['name'] ); break;
					case 'phone': $html .= form_phone( $element['label'], $element['value'], $element['size'], $element['required'], $element['name'] ); break;
					//TODO: Out-of-the-box elements - Later Work
					//case 'address': $element = form_address( $id ); break;
					//case 'price': $element = form_price( $id ); break;
					case 'date': $html .= form_date( $element['label'], $element['values'], $element['required'], $element['name'] ); break;
					case 'time': $html .= form_time( $element['label'], $element['values'], $element['required'], $element['name'] ); break;
					case 'textarea': $html .= form_textarea( $element['label'], ($element['value']), $element['size'], $element['required'], $element['name'] ); break;
					case 'dropdown': $html .= form_dropdown( $element['label'], explode(",",$element['values']), $element['size'], $element['required'], $element['name'] ); break;
					case 'checkbox': $html .= form_checkbox( $element['label'], explode(",",$element['values']), $element['required'], $element['name'] ); break;
					case 'radio': $html .= form_radio( $element['label'], explode(",",$element['values']), $element['required'], $element['name'] ); break;
					case 'fileupload': $html .= form_fileupload( $element['label'], $element['size'], $element['required'], $element['name'] ); break;
					//case 'buttons': $element = form_buttons( $id ); break;
				
				}
				
				$html .= "</li>";
				$field_keys[$element['name']] = $key;
			}
			
		}
		
		$html .= "<input type='hidden' name='field_keys' value='".serialize($field_keys)."' />";
		$html .= "<input class='submit' type='submit' value='Submmit' /><input type='reset' value='Reset' />";
		$html .= "<div class='response'></div>";
		
		return $html;

	}
	
	
	
	
	
	/*
	 * 
	 * name: post_data_filter() - Splash a new element to the screen
	 * @param none
	 * @return none
	 */	
	function create_element() { 
	
		// Require Our Helper library
		require_once( 'libraries/form.helper.php' );
		
		// Generate a new ID if no one was set
		$id = ( isset( $_POST['id'] ) ? $_POST['id'] : "elm_".uniqid() );
		
		
		
		switch( $_POST['element'] )
		{
			
			case 'paragraph': $element = form_mcearea( $id ); break;	
			case 'text': $element = form_text( $id ); break;
			case 'email': $element = form_email( $id ); break;
			case 'number': $element = form_number( $id ); break;
			case 'name': $element = form_name( $id ); break;
			case 'phone': $element = form_phone( $id ); break;
			//TODO: Out-of-the-box elements - Later Work
			//case 'address': $element = form_address( $id ); break;
			//case 'price': $element = form_price( $id ); break;
			case 'date': $element = form_date( $id ); break;
			case 'time': $element = form_time( $id ); break;
			case 'textarea': $element = form_textarea( $id ); break;
			case 'dropdown': $element = form_dropdown( $id ); break;
			case 'checkbox': $element = form_checkbox( $id ); break;
			case 'radio': $element = form_radio( $id ); break;
			case 'fileupload': $element = form_fileupload( $id ); break;
			//case 'buttons': $element = form_buttons( $id ); break;
				
		}
		
		// Label by default
		$label = 'Click to Edit this Label';

		$output = "<li id='$id' class='{$_POST['element']} state-default'><div class='element-head'>";
		$output .= ( $_POST['element'] == 'paragraph' ) ? '' : "<input type='text' class='label' name='elements[$id][label]' value='$label' /> <span class='info'></span> <img class='edit button-box' src='".WP_FORMS_ICONS."/tag_blue_edit.png' alt='Edit Options' title='Advanced Options' />";
		$output .= "<img class='remove button-box' title='Remove' src='".WP_FORMS_ICONS."/delete.png'' alt='Remove' />";
		$output .= "</div>";
		$output .= form_add_advanced_options( $id, $id, $_POST['element'] );
		$output .= "<div id='element_$id' class='element'>$element</div>";
		$output .= "<input type='hidden' class='type' name='elements[$id][type]' value='{$_POST['element']}'/>";
		$output .= "<input type='hidden' class='size' name='elements[{$id}][size]' value='medium'/>";
		$output .= "<input type='hidden' class='required' name='elements[{$id}][required]' value='required:false'/>";
		$output .= "<input type='hidden' class='name' name='elements[{$id}][name]' value='$id'/>";
		$output .= "</li>";
		
		die( $output );
		
	}







	/*
	 * 
	 * name: retrieve_elements() - Splash on admin screen elements that were saved before
	 * @param $elements
	 * @return $html
	 */	
	function retrieve_elements( $elements ) {
		
		// Requires Form Helper library
		require_once("libraries/form.helper.php");
		
		if ( ! is_array( $elements ) )
			return false;
			
		
		$html = "";
		
		
		foreach( $elements as $id => $element )
		{
			
			$label = isset( $element['label'] ) ? $element['label'] : false; 		
			$type = $element['type'];
			$name = $element['name'];
			$required = $element['required'];
			$size = $element['size'];
			$value = isset( $element['value'] ) ? $element['value'] : false;
			$values = isset( $element['values'] ) ? $element['values'] : false;
			
			
			switch( $element['type'] )
			{
			
				case 'paragraph': $elm = form_mcearea( $id, $value, $size ); break;
				case 'text': $elm = form_text( $id, $value, $size); break;
				case 'email': $elm = form_email( $id, $value, $size); break;
				case 'number': $elm = form_number( $id, $value, $size); break;
				case 'name': $elm = form_name( $id, $values, $size ); break;
				case 'phone': $elm = form_phone( $id, $value, $size ); break;
				//TODO: Out-of-the-box elements - Later Work
				//case 'address': $element = form_address( $id ); break;
				//case 'price': $element = form_price( $id ); break;
				case 'date': $elm = form_date( $id, $values, $size ); break;
				case 'time': $elm = form_time( $id, $values, $size ); break;
				case 'textarea': $elm = form_textarea( $id, $value, $size ); break;
				case 'dropdown': $elm = form_dropdown( $id, $value, explode(",",$values), $size ); break;
				case 'checkbox': $elm = form_checkbox( $id, explode(",",$values), $size ); break;
				case 'radio': $elm = form_radio( $id, $value, explode(",",$values), $size ); break;
				case 'fileupload': $elm = form_fileupload( $id, $value, $size ); break;
				//case 'buttons': $element = form_buttons( $id ); break;
				
			}	
		
			$html .= "<li id='{$id}' class='state-default'><div class='element-head'>";
			$html .= ( $type == 'paragraph' ) ? '' : "<input type='text' class='label' name='elements[{$id}][label]' value='$label' /> <img class='edit button-box' src='".WP_FORMS_ICONS."/tag_blue_edit.png' alt='Edit Options' title='Advanced Options' />";
			$html .= "<img class='remove button-box' title='Remove' src='".WP_FORMS_ICONS."/delete.png'' alt='Remove' />";
			$html .= "</div>";
			$html .= form_add_advanced_options( $id, $name, $type, $size, $required, $values );
			$html .= "<div id='element_{$id}' class='element'>$elm</div>";
			$html .= "<input type='hidden' class='type' name='elements[{$id}][type]' value='$type'/>";
			$html .= "<input type='hidden' class='size' name='elements[{$id}][size]' value='$size'/>";
			$html .= "<input type='hidden' class='required' name='elements[{$id}][required]' value='$required'/>";
			//$html .= "<input type='hidden' class='values' name='elements[{$id}][values]' value='$values'/>";
			$html .= "<input type='hidden' class='name' name='elements[{$id}][name]' value='$name'/>";
			$html .= "</li>";
			
		}
		
		return $html;

	}






	/*
	 * 
	 * name: main_box() - Central box where the form is built
	 * @param none
	 * @return none
	 */	
	function main_box() {
		
		global $post;
		
		
		$custom = get_post_custom($post->ID);
		
		$raw = ( isset( $custom['_elements'][0] ) ? unserialize($custom['_elements'][0]) : false );
		$order = ( isset( $custom['_elements_order'][0] ) ? $custom['_elements_order'][0] : false );
				
?>

		<div id="form_builder_panel">

				<fieldset class=''>
					<ul id="sortable">
					
						<p>Add Elements here by choosing from Form Elements box.</p>
					
						<?php if(!empty($raw)): ?>
										
							<?php echo $this->retrieve_elements( $raw ); ?>
							
						<?php endif; ?>
					</ul>
				</fieldset>

		</div>
		
		<input type="hidden" id="elements_order" name="elements_order" value="<?php echo $order; ?>" />

<?php

	}




	/*
	 * 
	 * name: options_box() - Options Box - Email, URL and Method
	 * @param none
	 * @return none
	 */	
	function options_box() {
		
		global $post;
		$custom = get_post_custom($post->ID);
		
				
		$methods = array( "post", "get" );
		
?>

		<p>The form will be processed through AJAX which do not require the browser to reload or to redirect the location. You Have two options:</p>
		
		<ol>
		
		<li>

		<label for="form_email">&nbsp;&nbsp;&nbsp;&nbsp;Send form data to this Email: 
			<input id="form_email" name="_form_email" type="text" value="<?php if(isset($custom['_form_email'][0])) echo $custom['_form_email'][0]; ?>" style="width:60%" />
		</label>
		
		</li>
		
		<p style="font-size:1.5em;"> &mdash; OR &mdash; </p>
		
		<li>
		
		<p>Request this location to process the form data:</p>

		<label for="form_action">&nbsp;&nbsp;&nbsp;&nbsp;URL:
			<input id="form_action" name="_form_action" type="text" value="<?php if(isset($custom['_form_action'][0])) echo $custom['_form_action'][0]; ?>" style="width:63%" />
		</label>
		<label for="form_method">&nbsp;&nbsp;Method:
			<select id="form_method" name="_form_method" >
				<?php foreach( $methods as $method ) : ?>
				<?php $selected = (isset($custom['_form_method'][0]) AND $method == $custom['_form_method'][0] ) ? "selected='selected' " : ""; ?>
					<option value="<?php echo $method; ?>"<?php echo $selected; ?>><?php echo strtoupper($method); ?></option>
				<?php endforeach; ?>
			</select>
		</label>
		<br />
		
		</li>
		
		</ol>
		

<?php

	}
	
	
	



	/*
	 * 
	 * name: side_elements() - Element List
	 * @param none
	 * @return none
	 */	
	function side_elements() {
		
?>
		<div id='form_builder_toolbox'>
				
			<ul>
				<li id='paragraph' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/layout_content.png" /><span>Content Section</span></li>
				<li id='text' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/textfield.png" /><span>Single Line Text Box</span></li>
				<li id='email' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/email.png" />Email Box</li>
				<li id='number' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/textfield_number.png" />Single Line Number Box</li>
				<li id='name' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/group.png" />Name Box</li>
				<li id='phone' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/bell.png" />Phone Box</li>
				<li id='date' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/date.png" />Date Picker Box</li>
				<li id='time' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/time.png" />Time Box</li>
				<li id='textarea' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/page_edit.png" />Paragraph Text Area</li>
				<li id='dropdown' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/dropdown.png" />Drop Down Box</li>
				<li id='checkbox' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/checkbox.png" />Check Boxes</li>
				<li id='radio' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/radio.png" />Multiple Choices</li>
				<li id='fileupload' class='toolbox'><img src="<?php echo WP_FORMS_ICONS; ?>/page_attach.png" />File Upload</li>
				<!--<li id='buttons' class='toolbox'>Buttons</li>-->
				<br class='clear' />	
			</ul>
			<br class='clear' />		
		</div>
		<br class='clear' />		
<?php

	}

}

// Init Forms Plugin
add_action("init", "WP_Form_Init");

function WP_Form_Init() { 
	global $WP_Form; 
	$WP_Form = new WP_Form();
	
}

?>
